%HW3, PHYS 3317
%tunneling through a barrier in phase space

hbar=1.054571596;              %Planck's constant (x10^34 J s)
echarge=1.602176462;		   %electron charge (x10^19 C)
emass=9.10938188;              %electron mass (x10^31 kg)

% prepare potential
len = 45;	                     %length of well (nm)
npoints=450;					 %number of sample points
x=linspace(0,len,npoints+1)';	 %position of sample points
dx = x(2)-x(1);
num_sol=160;                      %number of solutions sought

v = zeros(size(x));              %potential (eV)
v(x >= 20 & x <= 25) = 0.3;

v0 = zeros(size(x));             %zero potential

[energy,phi,H]=schrod(len,npoints,v,num_sol);		    %call Schrod.
[energy0,phi0,H0]=schrod(len,npoints,v0,num_sol);		%call Schrod.

% some basic operators: x-operator and k-operator
xop = diag(x);
kop = (diag(ones(npoints,1),1)-diag(ones(npoints,1),-1))/(2i*dx);

%setup the initial state and normalize
x0 = 15; vel0 = 0.3;
psi = exp(-(x-x0).^2./(2*1^2)).*exp(1i*emass*vel0*x/hbar); psi = psi/sqrt(psi'*psi);
fprintf('energy of the state: %g eV\n', psi(2:end-1)'*H*psi(2:end-1));
fprintf('dispersion of the state squared: %g nm^2\n', real(psi'*xop^2*psi) - real(psi'*xop*psi)^2);

%decompose into eigenstates
c_old = phi'*psi;
c0_old = phi0'*psi;

fprintf('1 - sum |c(i)|^2 = %g (ideally 0)\n', 1 - sum(abs(c_old).^2));

figure(1)
loglog(energy, abs(c_old).^2, '.'); % decomposition of the packet
xlabel('energy of j(th)-mode (eV)'); ylabel('coefficient |c_j|^2');

t = 0:1:80; % time in fs

% this is just to get momentum axis
[w px] = wig2(x, psi);
[pp,xx] = ndgrid(px,x); % momentum position Weyl operators
vx = px/emass;
idx = 1024/2-200:1024/2+200; % which vx range to plot
dp = px(2)-px(1);
%rr = [min(x(xi)) max(x(xi)) min(vx(idx)) max(vx(idx))];

for k = 1:length(t)
    % compute new state
    c = c_old .* exp(-1i*energy*t(k)/hbar);
    c0 = c0_old .* exp(-1i*energy0*t(k)/hbar);
    cmat = repmat(c, [1 npoints+1]).';
    cmat0 = repmat(c0, [1 npoints+1]).';
    psi_t = sum(cmat.*phi,2);
    psi_t0 = sum(cmat0.*phi0,2);

    % computer wigner
    [w ~] = wig2(x, psi_t);
    [w0 ~] = wig2(x, psi_t0);

    % compute averages
    xav = sum(w(:).*xx(:)*dx*dp);
    pav = sum(w(:).*pp(:)*dx*dp);
    emit = sqrt(sum(w(:).*(xx(:).^2-xav^2)*dx*dp)*...
                sum(w(:).*(pp(:).^2-pav^2)*dx*dp)-...
                sum(w(:).*(xx(:)-xav).*(pp(:)-pav)*dx*dp)^2);

    xav0 = sum(w0(:).*xx(:)*dx*dp);
    pav0 = sum(w0(:).*pp(:)*dx*dp);
    emit0 = sqrt(sum(w0(:).*(xx(:).^2-xav0^2)*dx*dp)*...
                sum(w0(:).*(pp(:).^2-pav0^2)*dx*dp)-...
                sum(w0(:).*(xx(:)-xav0).*(pp(:)-pav0)*dx*dp)^2);

    clf; figure(2)
    subplot(2,1,1); % show tunneling packet

    imagesc(x, vx(idx), w(idx,:)*pi*hbar); hold on;
    xlabel('distance (nm)'); ylabel('velocity (nm/fs)');
    title(sprintf('Tunneling: t = %.2gfs, <x> = %.3gnm, <v> =%.2gnm/fs, e=%.2ghbar', t(k), xav, pav/emass, emit/hbar));
    axis xy; caxis([-1 1]); colormap(bluewhitered(256));
    grid on; colorbar;
    plot([20 20], [vx(idx(1)) vx(idx(end))],'k');
    plot([25 25], [vx(idx(1)) vx(idx(end))],'k');
    plot(x, sum(w, 1)*dp+vx(idx(1)), 'k-', 10*sum(w(idx,:), 2)'*dx, vx(idx), 'k-')
    hold off;
    

    subplot(2,1,2); % show free propagating packet
    imagesc(x, vx(idx), w0(idx,:)*pi*hbar);
    title(sprintf('Free prop.: t = %.2gfs, <x> = %.3gnm, <v> =%.2gnm/fs, e=%.2ghbar', t(k), xav0, pav0/emass, emit0/hbar));
    xlabel('distance (nm)'); ylabel('velocity (nm/fs)');
    axis xy; caxis([-1 1]); colormap(bluewhitered(256));
    grid on; colorbar;
    hold on;
    plot(x, sum(w0, 1)*dp+vx(idx(1)), 'k-', 10*sum(w0(idx,:), 2)'*dx, vx(idx), 'k-')
    hold off;

    drawnow;
    %pause();
end